@@ -1,52 +1,45 @@
-//**********************************************************************;
-// Copyright (c) 2017, Intel Corporation
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice,
-// this list of conditions and the following disclaimer.
-//
-// 2. Redistributions in binary form must reproduce the above copyright notice,
-// this list of conditions and the following disclaimer in the documentation
-// and/or other materials provided with the distribution.
-//
-// 3. Neither the name of Intel Corporation nor the names of its contributors
-// may be used to endorse or promote products derived from this software without
-// specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
-// THE POSSIBILITY OF SUCH DAMAGE.
-//**********************************************************************;
+/* SPDX-License-Identifier: BSD-3-Clause */
+
 #include <errno.h>
 #include <inttypes.h>
+#include <libgen.h>
+#include <limits.h>
 #include <stdbool.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 
-#include <tss2/tss2_sys.h>
 #include <tss2/tss2_mu.h>
 
 #include "files.h"
 #include "log.h"
-#include "tpm2_util.h"
+#include "tpm2.h"
+#include "tpm2_tool.h"
+
+/**
+ * This is the magic for the file header. The header is organized
+ * as a big endian U32 (BEU32) of MAGIC followed by a BEU32 of the
+ * version number. Tools can define their own, individual file
+ * formats as they make sense, but they should always have the header.
+ */
+static const UINT32 MAGIC = 0xBADCC0DE;
+
+#define BAIL_ON_NULL(param, x) \
+    do { \
+        if (!x) { \
+            LOG_ERR(param" must be specified"); \
+            return false; \
+        } \
+    } while(0)
 
 bool files_get_file_size(FILE *fp, unsigned long *file_size, const char *path) {
 
     long current = ftell(fp);
     if (current < 0) {
         if (path) {
-            LOG_ERR("Error getting current file offset for file \"%s\" error: %s", path, strerror(errno));
+            LOG_ERR("Error getting current file offset for file \"%s\" error: "
+                    "%s", path, strerror(errno));
         }
         return false;
     }
@@ -54,7 +47,8 @@ bool files_get_file_size(FILE *fp, unsigned long *file_size, const char *path) {
     int rc = fseek(fp, 0, SEEK_END);
     if (rc < 0) {
         if (path) {
-            LOG_ERR("Error seeking to end of file \"%s\" error: %s", path, strerror(errno));
+            LOG_ERR("Error seeking to end of file \"%s\" error: %s", path,
+                    strerror(errno));
         }
         return false;
     }
@@ -70,18 +64,20 @@ bool files_get_file_size(FILE *fp, unsigned long *file_size, const char *path) {
     rc = fseek(fp, current, SEEK_SET);
     if (rc < 0) {
         if (path) {
-            LOG_ERR("Could not restore initial stream position for file \"%s\" failed: %s", path, strerror(errno));
+            LOG_ERR(
+                    "Could not restore initial stream position for file \"%s\" "
+                    "failed: %s", path, strerror(errno));
         }
         return false;
     }
 
     /* size cannot be negative at this point */
-    *file_size = (unsigned long)size;
+    *file_size = (unsigned long) size;
     return true;
 }
 
 static bool read_bytes_from_file(FILE *f, UINT8 *buf, UINT16 *size,
-                                 const char *path) {
+        const char *path) {
     unsigned long file_size;
     bool result = files_get_file_size(f, &file_size, path);
     if (!result) {
@@ -93,8 +89,8 @@ static bool read_bytes_from_file(FILE *f, UINT8 *buf, UINT16 *size,
     if (file_size > *size) {
         if (path) {
             LOG_ERR(
-                    "File \"%s\" size is larger than buffer, got %lu expected less than %u",
-                    path, file_size, *size);
+                    "File \"%s\" size is larger than buffer, got %lu expected "
+                    "less than %u", path, file_size, *size);
         }
         return false;
     }
@@ -131,11 +127,15 @@ bool files_load_bytes_from_path(const char *path, UINT8 *buf, UINT16 *size) {
 
 bool files_save_bytes_to_file(const char *path, UINT8 *buf, UINT16 size) {
 
-    if (!path || !buf) {
+    if (!buf) {
         return false;
     }
 
-    FILE *fp = fopen(path, "wb+");
+    if (!path && !output_enabled) {
+        return true;
+    }
+
+    FILE *fp = path ? fopen(path, "wb+") : stdout;
     if (!fp) {
         LOG_ERR("Could not open file \"%s\", error: %s", path, strerror(errno));
         return false;
@@ -145,7 +145,11 @@ bool files_save_bytes_to_file(const char *path, UINT8 *buf, UINT16 size) {
     if (!result) {
         LOG_ERR("Could not write data to file \"%s\"", path);
     }
-    fclose(fp);
+
+    if (fp != stdout) {
+        fclose(fp);
+    }
+
     return result;
 }
 
@@ -154,117 +158,118 @@ bool files_save_bytes_to_file(const char *path, UINT8 *buf, UINT16 size) {
  */
 #define CONTEXT_VERSION 1
 
-bool files_save_tpm_context_to_file(TSS2_SYS_CONTEXT *sysContext, TPM2_HANDLE handle,
-        const char *path) {
-
-    TPMS_CONTEXT context;
-
-    TSS2_RC rval = Tss2_Sys_ContextSave(sysContext, handle, &context);
-    if (rval != TPM2_RC_SUCCESS) {
-        LOG_ERR(
-                "Tss2_Sys_ContextSave: Saving handle 0x%x context failed. TPM Error:0x%x",
-                handle, rval);
-        return false;
-    }
-
-    FILE *f = fopen(path, "w+b");
-    if (!f) {
-        LOG_ERR("Error opening file \"%s\" due to error: %s", path,
-                strerror(errno));
-        return false;
-    }
+bool files_save_context(TPMS_CONTEXT *context, FILE *stream) {
 
     /*
      * Saving the TPMS_CONTEXT structure to disk, format:
      * TPM2.0-TOOLS HEADER
-     * U32 hiearchy
+     * U32 hierarchy
      * U32 savedHandle
      * U64 sequence
      * U16 contextBlobLength
      * BYTE[] contextBlob
      */
-    bool result = files_write_header(f, CONTEXT_VERSION);
+    bool result = files_write_header(stream, CONTEXT_VERSION);
     if (!result) {
-        LOG_ERR("Could not write header for file: \"%s\"", path);
+        LOG_ERR("Could not write context file header");
         goto out;
     }
 
     // UINT32
-    result = files_write_32(f, context.hierarchy);
+    result = files_write_32(stream, context->hierarchy);
     if (!result) {
-        LOG_ERR("Could not write hierarchy for file: \"%s\"", path);
+        LOG_ERR("Could not write hierarchy");
         goto out;
     }
 
-    result = files_write_32(f, context.savedHandle);
+    result = files_write_32(stream, context->savedHandle);
     if (!result) {
-        LOG_ERR("Could not write savedHandle for file: \"%s\"", path);
+        LOG_ERR("Could not write savedHandle");
         goto out;
     }
+    LOG_INFO("Save TPMS_CONTEXT->savedHandle: 0x%x", context->savedHandle);
 
     // UINT64
-    result = files_write_64(f, context.sequence);
+    result = files_write_64(stream, context->sequence);
     if (!result) {
-        LOG_ERR("Could not write sequence for file: \"%s\"", path);
+        LOG_ERR("Could not write sequence");
         goto out;
     }
 
     // U16 LENGTH
-    result = files_write_16(f, context.contextBlob.size);
+    result = files_write_16(stream, context->contextBlob.size);
     if (!result) {
-        LOG_ERR("Could not write contextBob size file: \"%s\"", path);
+        LOG_ERR("Could not write contextBob size");
         goto out;
     }
 
     // BYTE[] contextBlob
-    result = files_write_bytes(f, context.contextBlob.buffer,
-            context.contextBlob.size);
+    result = files_write_bytes(stream, context->contextBlob.buffer,
+            context->contextBlob.size);
     if (!result) {
-        LOG_ERR("Could not write contextBlob buffer for file: \"%s\"", path);
+        LOG_ERR("Could not write contextBlob buffer");
     }
     /* result is set by file_write_bytes() */
 
 out:
-    fclose(f);
     return result;
 }
 
-bool files_load_tpm_context_from_file(TSS2_SYS_CONTEXT *sapi_context,
-        TPM2_HANDLE *handle, const char *path) {
+tool_rc files_save_tpm_context_to_file(ESYS_CONTEXT *ectx, ESYS_TR handle,
+        FILE *stream) {
 
-    TSS2_RC rval;
+    TPMS_CONTEXT *context = NULL;
 
-    FILE *f = fopen(path, "rb");
+    tool_rc rc = tpm2_context_save(ectx, handle, &context);
+    if (rc != tool_rc_success) {
+        return rc;
+    }
+
+    bool result = files_save_context(context, stream);
+    free(context);
+    return result ? tool_rc_success : tool_rc_general_error;
+}
+
+tool_rc files_save_tpm_context_to_path(ESYS_CONTEXT *context, ESYS_TR handle,
+        const char *path) {
+
+    FILE *f = fopen(path, "w+b");
     if (!f) {
         LOG_ERR("Error opening file \"%s\" due to error: %s", path,
                 strerror(errno));
-        return false;
+        return tool_rc_general_error;
     }
 
+    tool_rc rc = files_save_tpm_context_to_file(context, handle, f);
+    fclose(f);
+    return rc;
+}
+
+static bool load_tpm_context_file(FILE *fstream, TPMS_CONTEXT *context) {
+
     /*
      * Reading the TPMS_CONTEXT structure to disk, format:
      * TPM2.0-TOOLS HEADER
-     * U32 hiearchy
+     * U32 hierarchy
      * U32 savedHandle
      * U64 sequence
      * U16 contextBlobLength
      * BYTE[] contextBlob
      */
     UINT32 version;
-    TPMS_CONTEXT context;
-    bool result = files_read_header(f, &version);
+    bool result = files_read_header(fstream, &version);
     if (!result) {
-        LOG_WARN(
-                "The tpm context file \"%s\" does not appear in the proper format, assuming old format, this will be converted on the next save.",
-                path);
-        rewind(f);
-        result = files_read_bytes(f, (UINT8 *) &context, sizeof(context));
+        LOG_WARN("The loaded tpm context does not appear to be in the proper "
+                "format, assuming old format, this will be converted on the "
+                "next save.");
+        rewind(fstream);
+        result = files_read_bytes(fstream, (UINT8 *) context, sizeof(*context));
         if (!result) {
-            LOG_ERR("Could not load file \"%s\" into tpm context", path);
+            LOG_ERR("Could not load tpm context file");
             goto out;
         }
         /* Success load the context into the TPM */
-        goto load_to_tpm;
+        goto out;
     }
 
     if (version != CONTEXT_VERSION) {
@@ -274,61 +279,148 @@ bool files_load_tpm_context_from_file(TSS2_SYS_CONTEXT *sapi_context,
         goto out;
     }
 
-    result = files_read_32(f, &context.hierarchy);
+    result = files_read_32(fstream, &context->hierarchy);
     if (!result) {
         LOG_ERR("Error reading hierarchy!");
         goto out;
     }
 
-    result = files_read_32(f, &context.savedHandle);
+    result = files_read_32(fstream, &context->savedHandle);
     if (!result) {
         LOG_ERR("Error reading savedHandle!");
         goto out;
     }
+    LOG_INFO("load: TPMS_CONTEXT->savedHandle: 0x%x", context->savedHandle);
 
-    result = files_read_64(f, &context.sequence);
+    result = files_read_64(fstream, &context->sequence);
     if (!result) {
         LOG_ERR("Error reading sequence!");
         goto out;
     }
 
-    result = files_read_16(f, &context.contextBlob.size);
+    result = files_read_16(fstream, &context->contextBlob.size);
     if (!result) {
         LOG_ERR("Error reading contextBlob.size!");
         goto out;
     }
 
-    if (context.contextBlob.size > sizeof(context.contextBlob.buffer)) {
-        LOG_ERR(
-                "Size mismatch found on contextBlob, got %"PRIu16" expected less than or equal to %zu",
-                context.contextBlob.size,
-                sizeof(context.contextBlob.buffer));
+    if (context->contextBlob.size > sizeof(context->contextBlob.buffer)) {
+        LOG_ERR("Size mismatch found on contextBlob, got %"PRIu16" expected "
+                "less than or equal to %zu", context->contextBlob.size,
+                sizeof(context->contextBlob.buffer));
         result = false;
         goto out;
     }
 
-    result = files_read_bytes(f, context.contextBlob.buffer,
-            context.contextBlob.size);
+    result = files_read_bytes(fstream, context->contextBlob.buffer,
+            context->contextBlob.size);
     if (!result) {
         LOG_ERR("Error reading contextBlob.size!");
         goto out;
     }
 
-load_to_tpm:
-    rval = Tss2_Sys_ContextLoad(sapi_context, &context, handle);
-    if (rval != TPM2_RC_SUCCESS) {
-        LOG_ERR("ContextLoad Error. TPM Error:0x%x", rval);
-        result = false;
-        goto out;
-    }
-
-    result = true;
-
 out:
-    fclose(f);
     return result;
 }
 
+static bool check_magic(FILE *fstream, bool seek_reset) {
+
+    BAIL_ON_NULL("FILE", fstream);
+    UINT32 magic = 0;
+    bool res = files_read_32(fstream, &magic);
+    if (!res) {
+        return false;
+    }
+
+    bool match = magic == MAGIC;
+
+    if (seek_reset) {
+        int rc = fseek(fstream, -sizeof(magic), SEEK_CUR);
+        if (rc != 0) {
+            LOG_ERR("fseek failed: %s", strerror(errno));
+            return false;
+        }
+        return match;
+    }
+
+    if (!match) {
+        LOG_ERR("Found magic 0x%x did not match expected magic of 0x%x!", magic,
+                MAGIC);
+    }
+
+    return match;
+}
+
+tool_rc files_load_tpm_context_from_file(ESYS_CONTEXT *context,
+        ESYS_TR *tr_handle, FILE *fstream) {
+
+    TPMS_CONTEXT tpms_context;
+    tool_rc rc = tool_rc_general_error;
+
+    bool result = check_magic(fstream, true);
+    if (result) {
+        LOG_INFO("Assuming tpm context file");
+        result = load_tpm_context_file(fstream, &tpms_context);
+        if (!result) {
+            LOG_ERR("Failed to load_tpm_context_file()");
+            goto out;
+        }
+
+        return tpm2_context_load(context, &tpms_context, tr_handle);
+    }
+
+    ESYS_TR loaded_handle;
+    LOG_INFO("Assuming tpm context file");
+    /* try ESYS TR deserialize */
+    unsigned long size = 0;
+    result = files_get_file_size(fstream, &size, NULL);
+    if (!result) {
+        LOG_ERR("Failed to get file size: %s", strerror(ferror(fstream)));
+        goto out;
+    }
+
+    if (size < 1) {
+        LOG_ERR("Invalid serialized ESYS_TR size, got: %lu", size);
+        goto out;
+    }
+
+    uint8_t *buffer = calloc(1, size);
+    if (!buffer) {
+        LOG_ERR("oom");
+        goto out;
+    }
+
+    result = files_read_bytes(fstream, buffer, size);
+    if (!result) {
+        LOG_ERR("Could not read serialized ESYS_TR from disk");
+        free(buffer);
+        goto out;
+    }
+
+    rc = tpm2_tr_deserialize(context, buffer, size, &loaded_handle);
+    free(buffer);
+    if (rc == tool_rc_success) {
+        *tr_handle = loaded_handle;
+    }
+    out: return rc;
+}
+
+tool_rc files_load_tpm_context_from_path(ESYS_CONTEXT *context,
+        ESYS_TR *tr_handle, const char *path) {
+
+    FILE *f = fopen(path, "rb");
+    if (!f) {
+        LOG_WARN("Error opening file \"%s\" due to error: %s", path,
+                strerror(errno));
+        return false;
+    }
+
+    tool_rc rc = files_load_tpm_context_from_file(context, tr_handle, f);
+
+    fclose(f);
+    return rc;
+}
+
 bool files_does_file_exist(const char *path) {
 
     if (!path) {
@@ -336,10 +428,10 @@ bool files_does_file_exist(const char *path) {
         return false;
     }
 
-    FILE *fp = fopen(path,"rb");
+    FILE *fp = fopen(path, "rb");
     if (fp) {
         fclose(fp);
-        LOG_ERR("Path: %s already exists. Please rename or delete the file!",
+        LOG_WARN("Path: %s already exists. Please rename or delete the file!",
                 path);
         return true;
     }
@@ -360,8 +452,8 @@ bool files_get_file_size_path(const char *path, unsigned long *file_size) {
         return false;
     }
 
-    FILE *fp = fopen(path,"rb");
-    if(!fp) {
+    FILE *fp = fopen(path, "rb");
+    if (!fp) {
         LOG_ERR("Could not open file: \"%s\" error: %s", path, strerror(errno));
         return false;
     }
@@ -372,14 +464,6 @@ bool files_get_file_size_path(const char *path, unsigned long *file_size) {
     return result;
 }
 
-/**
- * This is the magic for the file header. The header is organized
- * as a big endian U32 (BEU32) of MAGIC followed by a BEU32 of the
- * version number. Tools can define their own, individual file
- * formats as they make sense, but they should always have the header.
- */
-static const UINT32 MAGIC = 0xBADCC0DE;
-
 /**
  * Writes size bytes to a file, continuing on EINTR short writes.
  * @param f
@@ -440,14 +524,6 @@ static bool readx(FILE *f, UINT8 *data, size_t size) {
     return true;
 }
 
-#define BAIL_ON_NULL(param, x) \
-    do { \
-        if (!x) { \
-            LOG_ERR(param" must be specified"); \
-            return false; \
-        } \
-    } while(0)
-
 #define BE_CONVERT(value, size) \
     do { \
         if (!tpm2_util_is_big_endian()) { \
@@ -515,64 +591,96 @@ bool files_read_header(FILE *out, uint32_t *version) {
     BAIL_ON_NULL("FILE", out);
     BAIL_ON_NULL("version", version);
 
-    UINT32 magic;
-    bool res = files_read_32(out, &magic);
-    if (!res) {
-        return false;
-    }
-
-    if (magic != MAGIC) {
-        LOG_ERR("Found magic 0x%x did not match expected magic of 0x%x!",
-                magic, MAGIC);
+    bool result = check_magic(out, false);
+    if (!result) {
         return false;
     }
 
     return files_read_32(out, version);
 }
 
-bool files_load_bytes_from_file_or_stdin(const char *path, UINT16 *size, BYTE *buf) {
-
-    FILE *file =  path ? fopen(path, "rb") : stdin;
-    path = file != stdin ? path : "<stdin>";
-    if (!file) {
-        LOG_ERR("Could not open file: \"%s\", error: %s", path,
-                strerror(errno));
-        return false;
-    }
-
+bool files_load_bytes_from_buffer_or_file_or_stdin(char *input_buffer,
+        const char *path, UINT16 *size, BYTE *buf) {
     /*
-     * Attempt to accurately read the file based on the file size.
-     * This may fail on stdin when it's a pipe.
+     * Read from stdin where in size is not fixed or is unknown
+     * Size however cannot be bigger than UINT16_MAX
      */
-    if (file == stdin) {
-        path = NULL;
+    bool is_stdin_read_complete = false;
+    if (!input_buffer && !path && !*size) {
+        while (!is_stdin_read_complete) {
+            *size += fread(buf, 1, UINT16_MAX, stdin);
+            if (feof(stdin)) {
+                is_stdin_read_complete = true;
+            }
+            if (ferror(stdin)) {
+                LOG_ERR("Failed read from stdin.");
+                return false;
+            }
+            if (*size >= UINT16_MAX) {
+                LOG_ERR("Stdin input bigger than UINT16_MAX");
+                return false;
+            }
+        }
+        return true;
     }
 
     UINT16 original_size = *size;
-    bool res = files_load_bytes_from_path(path, buf,
-            size);
-    if (!res) {
-        res = true;
-        *size = fread(buf, 1,
-                *size, file);
-        if (!feof(file)) {
-            LOG_ERR("Data to be sealed larger than expected. Got %u expected %u",
-                    original_size, res);
-            res = false;
+    bool res;
+    if (input_buffer) {
+        size_t input_buffer_size = strlen(input_buffer);
+        if (path) {
+            LOG_ERR("Specify either the input buffer or file path to load data,"
+                    " not both");
         }
-        else if (ferror(file)) {
-            LOG_ERR("Error reading sealed data from \"<stdin>\"");
+        if (input_buffer_size != (size_t) original_size) {
+            LOG_ERR("Unexpected data size. Got %u expected %u", original_size,
+                    (unsigned )input_buffer_size);
             res = false;
+        } else {
+            memcpy(buf, input_buffer, input_buffer_size);
+            res = true;
+        }
+    } else {
+        if (path) {
+            res = files_load_bytes_from_path(path, buf, size);
+        } else {
+            /*
+             * Read from stdin where in size is fixed or known
+             * Size however cannot be bigger than UINT16_MAX
+             * Attempt to accurately read the file based on the file size.
+             * This may fail on stdin when it's a pipe.
+             */
+            res = read_bytes_from_file(stdin, buf, size, path);
+            if (!res) {
+                *size = fread(buf, 1, *size, stdin);
+                if (ferror(stdin)) {
+                    LOG_ERR("Error reading data from \"<stdin>\"");
+                    res = false;
+                } else {
+                    res = true;
+                }
+            } else {
+                res = true;
+            }
         }
     }
-
-    if (file != stdin) {
-        fclose(file);
-    }
-
     return res;
 }
 
+tool_rc files_save_ESYS_TR(ESYS_CONTEXT *ectx, ESYS_TR handle, const char *path) {
+
+    size_t size;
+    uint8_t *buffer;
+    tool_rc rc = tpm2_tr_serialize(ectx, handle, &buffer, &size);
+    if (rc != tool_rc_success) {
+        return rc;
+    }
+
+    bool result = files_save_bytes_to_file(path, buffer, size);
+    free(buffer);
+    return result ? tool_rc_success : tool_rc_general_error;
+}
+
 #define SAVE_TYPE(type, name) \
     bool files_save_##name(type *name, const char *path) { \
     \
@@ -601,15 +709,13 @@ bool files_load_bytes_from_file_or_stdin(const char *path, UINT16 *size, BYTE *b
         TSS2_RC rc = Tss2_MU_##type##_Unmarshal(buffer, size, &offset, name); \
         if (rc != TSS2_RC_SUCCESS) { \
             LOG_ERR("Error serializing "str(name)" structure: 0x%x", rc); \
+            LOG_ERR("The input file needs to be a valid "xstr(type)" data structure"); \
             return false; \
         } \
         \
-        return rc == TSS2_RC_SUCCESS; \
+        return rc == TPM2_RC_SUCCESS; \
     }
 
-SAVE_TYPE(TPM2B_PRIVATE, private)
-LOAD_TYPE(TPM2B_PRIVATE, private)
-
 SAVE_TYPE(TPM2B_PUBLIC, public)
 LOAD_TYPE(TPM2B_PUBLIC, public)
 
@@ -624,3 +730,9 @@ LOAD_TYPE(TPM2B_SENSITIVE, sensitive)
 
 SAVE_TYPE(TPMT_TK_HASHCHECK, validation)
 LOAD_TYPE(TPMT_TK_HASHCHECK, validation)
+
+SAVE_TYPE(TPM2B_PRIVATE, private)
+LOAD_TYPE(TPM2B_PRIVATE, private)
+
+SAVE_TYPE(TPM2B_ENCRYPTED_SECRET, encrypted_seed)
+LOAD_TYPE(TPM2B_ENCRYPTED_SECRET, encrypted_seed)
