@@ -1,40 +1,14 @@
-//**********************************************************************;
-// Copyright (c) 2017, Intel Corporation
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice,
-// this list of conditions and the following disclaimer.
-//
-// 2. Redistributions in binary form must reproduce the above copyright notice,
-// this list of conditions and the following disclaimer in the documentation
-// and/or other materials provided with the distribution.
-//
-// 3. Neither the name of Intel Corporation nor the names of its contributors
-// may be used to endorse or promote products derived from this software without
-// specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
-// THE POSSIBILITY OF SUCH DAMAGE.
-//**********************************************************************;
+/* SPDX-License-Identifier: BSD-3-Clause */
+
 #ifndef FILES_H
 #define FILES_H
 
 #include <stdbool.h>
 #include <stdio.h>
 
-#include <tss2/tss2_sys.h>
+#include <tss2/tss2_esys.h>
+
+#include "tool_rc.h"
 
 /**
  * Reads a series of bytes from a file as a byte array. This is similar to files_read_bytes(),
@@ -54,7 +28,9 @@
 bool files_load_bytes_from_path(const char *path, UINT8 *buf, UINT16 *size);
 
 /**
- * Loads data from a file path or stdin enforcing an upper bound on size.
+ * Loads data from an input buffer or file path or stdin enforcing an upper bound on size.
+ * @param input_buffer
+ *   The buffer to read the input data from, NULL means either specified by path or stdin
  * @param path
  *  The path to load data from, NULL means stdin.
  * @param size
@@ -64,11 +40,13 @@ bool files_load_bytes_from_path(const char *path, UINT8 *buf, UINT16 *size);
  * @return
  *  True on success or false otherwise.
  */
-bool files_load_bytes_from_file_or_stdin(const char *path, UINT16 *size, BYTE *buf);
+bool files_load_bytes_from_buffer_or_file_or_stdin(char *input_buffer,
+        const char *path, UINT16 *size, BYTE *buf);
 
 /**
  * Similar to files_write_bytes(), in that it writes an array of bytes to disk,
- * but this routine opens and closes the file on the callers behalf.
+ * but this routine opens and closes the file on the callers behalf. If the path
+ * is NULL and silent output has not been enabled, then stdout is used.
  * @param path
  *  The path to the file to write the data to.
  * @param buf
@@ -81,54 +59,76 @@ bool files_load_bytes_from_file_or_stdin(const char *path, UINT16 *size, BYTE *b
 bool files_save_bytes_to_file(const char *path, UINT8 *buf, UINT16 size);
 
 /**
- * Saves the TPM context for an object handle to disk by calling Tss2_Sys_ContextSave() and serializing the
- * resulting TPMS_CONTEXT structure to disk.
- * @param sapi_context
- *  The system api context
+ * Saves the TPM ESAPI context for an object handle to disk by calling
+ * ContextSave() and serializing the resulting TPMS_CONTEXT structure
+ * to disk.
+ * @param context
+ *  The Enhances System API (ESAPI) context
  * @param handle
  *  The object handle for the object to save.
  * @param path
  *  The output path of the file.
  *
  * @return
- *  True on success, False on error.
+ *  tool_rc indicating status.
  */
-bool files_save_tpm_context_to_file(TSS2_SYS_CONTEXT *sapi_context, TPM2_HANDLE handle, const char *path);
+tool_rc files_save_tpm_context_to_path(ESYS_CONTEXT *context, ESYS_TR handle,
+        const char *path);
 
 /**
- * Loads a TPM object context from disk.
- * @param sapi_context
- *  The system API context
+ * Like files_save_tpm_context_to_path() but saves a tpm session to a FILE stream.
+ * @param context
+ *  The Enhances System API (ESAPI) context
  * @param handle
- *  The object handle that was saved.
+ *  The object handle for the object to save.
+ * @param stream
+ *  The FILE stream to save too.
+ * @return
+ *  tool_rc indicating status.
+ */
+tool_rc files_save_tpm_context_to_file(ESYS_CONTEXT *context, ESYS_TR handle,
+        FILE *stream);
+
+/**
+ * Loads a ESAPI TPM object context from disk or an ESAPI serialized ESYS_TR object.
+ * @param context
+ *  The Enhanced System API (ESAPI) context
+ * @param tr_handle
+ *  Optional. The Esys handle for the TPM2 object.
  * @param path
  *  The path to the input file.
  * @return
- *  True on Success, false on error.
+ *  tool_rc status indicating success.
  */
-bool files_load_tpm_context_from_file(TSS2_SYS_CONTEXT *sapi_context, TPM2_HANDLE *handle, const char *path);
+tool_rc files_load_tpm_context_from_path(ESYS_CONTEXT *context,
+        ESYS_TR *tr_handle, const char *path);
 
 /**
- * Serializes a TPM2B_PPRIVATE to the file path provided.
- * @param private
- *  The TPM2B_PRIVATE to save to disk.
+ * Like files_load_tpm_context_from_path() but loads the context from a FILE stream.
+ * @param context
+ *  The Enhanced System API (ESAPI) context
+ * @param tr_handle
+ *  Optional. The Esys handle for the TPM2 object
+ * @param stream
+ *  The FILE stream to read from.
+ * @return
+ *  tool_rc status indicating success.
+ */
+tool_rc files_load_tpm_context_from_file(ESYS_CONTEXT *context,
+        ESYS_TR *tr_handle, FILE *stream);
+
+/**
+ * Save an ESYS_TR to disk.
+ * @param ectx
+ *  The ESAPI context
+ * @param handle
+ *  The handle to serialize.
  * @param path
  *  The path to save to.
  * @return
- *  true on success, false on error.
+ *  A tool_rc indicating status.
  */
-bool files_save_private(TPM2B_PRIVATE *private, const char *path);
-
-/**
- * Loads a TPM2B_PRIVATE from disk that was saved with files_save_private()
- * @param path
- *  The path to load from.
- * @param private
- *  The TPM2B_PRIVATE to load.
- * @return
- *  true on success, false on error.
- */
-bool files_load_private(const char *path, TPM2B_PRIVATE *private);
+tool_rc files_save_ESYS_TR(ESYS_CONTEXT *ectx, ESYS_TR handle, const char *path);
 
 /**
  * Serializes a TPM2B_PUBLIC to the file path provided.
@@ -207,6 +207,16 @@ bool files_load_ticket(const char *path, TPMT_TK_VERIFIED *ticket);
  */
 bool files_load_sensitive(const char *path, TPM2B_SENSITIVE *sensitive);
 
+/**
+ * Serializes a TPM2B_SENSITIVE to the file path provided.
+ * @param sensitive
+ *  The TPM2B_SENSITIVE to save to disk.
+ * @param path
+ *  The path to save to.
+ * @return
+ *  true on success, false on error.
+ */
+bool files_save_sensitive(TPM2B_SENSITIVE *sensitive, const char *path);
 /**
  * Serializes a TPMT_TK_HASHCHECK to the file path provided.
  * @param validation
@@ -229,6 +239,52 @@ bool files_save_validation(TPMT_TK_HASHCHECK *validation, const char *path);
  */
 bool files_load_validation(const char *path, TPMT_TK_HASHCHECK *validation);
 
+/**
+ * Serializes a TPM2B_PRIVATE to the file path provided.
+ * @param private
+ *  The TPM2B_PRIVATE to save to disk.
+ * @param path
+ *  The path to save to.
+ * @return
+ *  true on success, false on error.
+ */
+bool files_save_private(TPM2B_PRIVATE *private, const char *path);
+
+/**
+ * Loads a TPM2B_PRIVATE from disk.
+ * @param private
+ *  The path to load from.
+ * @param validation
+ *  The TPM2B_PRIVATE to load.
+ * @return
+ *  true on success, false on error.
+ */
+bool files_load_private(const char *path, TPM2B_PRIVATE *private);
+
+/**
+ * Serializes a TPM2B_ENCRYPTED_SECRET to the file path provided.
+ * @param encrypted_seed
+ *  The TPM2B_ENCRYPTED_SECRET to save to disk.
+ * @param path
+ *  The path to save to.
+ * @return
+ *  true on success, false on error.
+ */
+bool files_save_encrypted_seed(TPM2B_ENCRYPTED_SECRET *encrypted_seed,
+        const char *path);
+
+/**
+ * Loads a TPM2B_ENCRYPTED_SECRET from disk.
+ * @param encrypted_seed
+ *  The path to load from.
+ * @param validation
+ *  The TPM2B_ENCRYPTED_SECRET to load.
+ * @return
+ *  true on success, false on error.
+ */
+bool files_load_encrypted_seed(const char *path,
+        TPM2B_ENCRYPTED_SECRET *encrypted_seed);
+
 /**
  * Checks a file for existence.
  * @param path
@@ -242,7 +298,7 @@ bool files_does_file_exist(const char *path);
 /**
  * Retrieves a files size given a file path.
  * @param path
- *  The path of the file to retreive the size of.
+ *  The path of the file to retrieve the size of.
  * @param file_size
  *  A pointer to an unsigned long to return the file size. The
  *  pointed to value is valid only on a true return.
